@@ -120,16 +120,20 @@ list_drives (void)
 			 has_audio ? " (has-audio)" : "");
 
 		size = nautilus_burn_drive_get_media_size (drive);
-		g_print ("media size: %0.2f MiB", (float) size / 1024 / 1024);
+		if (size >= 0) {
+			g_print ("media size: %0.2f MiB", (float) size / 1024 / 1024);
 
-		if (media_type == NAUTILUS_BURN_MEDIA_TYPE_CD
-		    || media_type == NAUTILUS_BURN_MEDIA_TYPE_CDR
-		    || media_type == NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
-			g_print (" approx. or %d mins %d secs\n",
-				 NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size) / 60,
-				 NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size) % 60);
+			if (media_type == NAUTILUS_BURN_MEDIA_TYPE_CD
+			    || media_type == NAUTILUS_BURN_MEDIA_TYPE_CDR
+			    || media_type == NAUTILUS_BURN_MEDIA_TYPE_CDRW) {
+				g_print (" approx. or %d mins %d secs\n",
+					 NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size) / 60,
+					 NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size) % 60);
+			} else {
+				g_print ("\n");
+			}
 		} else {
-			g_print ("\n");
+			g_print ("media size: Could not be determined\n");
 		}
 
 		g_print ("CD-Recorder/SCSI devices only: max_speed_write: %d"
