@@ -24,10 +24,6 @@
  *          William Jon McCann <mccann@jhu.edu>
  */
 
-#include "mapping-protocol.h"
-#include "mapping-method.h"
-#include <glib/ghash.h>
-#include <glib/glist.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -39,6 +35,9 @@
 #include <sys/un.h>
 #include <sys/poll.h>
 
+#include <glib/ghash.h>
+#include <glib/glist.h>
+
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -49,6 +48,9 @@
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
 
+#include "mapping-protocol.h"
+#include "mapping-method.h"
+
 GIOChannel *daemon_ioc = NULL;
 
 G_LOCK_DEFINE_STATIC (mapping_lock);
@@ -426,8 +428,18 @@ do_close (GnomeVFSMethod       *method,
 	  GnomeVFSContext      *context)
 
 {
+	VirtualFileHandle *vfh;
+	GnomeVFSHandle    *handle;
+
 	DEBUG_PRINT ("do_close: %p\n", method_handle);
-	return gnome_vfs_close_cancellable (((VirtualFileHandle *)method_handle)->handle, context);
+
+	vfh    = (VirtualFileHandle *)method_handle;
+	handle = vfh->handle;
+
+	g_free (vfh->backing_file);
+	g_free (vfh);
+
+	return gnome_vfs_close_cancellable (handle, context);
 }
 
 static GnomeVFSResult
