@@ -80,6 +80,14 @@ int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_bla
 
 #define CD_ROM_SPEED 176
 
+#define CDS_NO_INFO		0	/* if not implemented */
+#define CDS_AUDIO		100
+#define CDS_DATA_1		101
+#define CDS_DATA_2		102
+#define CDS_XA_2_1		103
+#define CDS_XA_2_2		104
+#define CDS_MIXED		105
+
 static struct {
 	const char *name;
 	gboolean can_write_cdr;
@@ -415,11 +423,11 @@ static int
 get_disc_type (const char *dev_path)
 {
 	int                   fd = 0;
+	int                   type;
 #if defined (HAVE_SYS_MNTTAB_H)   /* Solaris */
 	GString              *new_dev_path;
 	struct cdrom_tocentry entry;
 	struct cdrom_tochdr   header;
-	int                   type;
 
 	/* For ioctl call to work dev_path has to be /vol/dev/rdsk.
 	 * There has to be a nicer way to do this.
@@ -433,7 +441,7 @@ get_disc_type (const char *dev_path)
 		return -1;
 	}
 
-	if (ioctl (fd, CDROMREADTOCHDR, &header) == 0) {
+	if (ioctl (fd, CDROMREADTOCHDR, &header) < 0) {
 		return -1;
 	}
 
@@ -444,7 +452,7 @@ get_disc_type (const char *dev_path)
 	     entry.cdte_track++) {
 		entry.cdte_format = CDROM_LBA;
 		if (ioctl (fd, CDROMREADTOCENTRY, &entry) == 0) {
-			if (entry.cdte_ctrl & CDROM_DATA_TRACK) {
+			if (!(entry.cdte_ctrl & CDROM_DATA_TRACK)) {
 				type = CDS_AUDIO;
 				break;
 			}
@@ -473,7 +481,8 @@ get_disc_type (const char *dev_path)
 	    	return -1;
 	}
 
-	if (ioctl (fd, CDIOREADTOCHEADER, &header) == 0) {
+	if (ioctl (fd, CDIOREADTOCHEADER, &header) < 0) {
+		close (fd);
 	    	return -1;
 	}
 
@@ -484,7 +493,7 @@ get_disc_type (const char *dev_path)
 		entry.track++) {
 	    	entry.address_format = CD_LBA_FORMAT;
 		if (ioctl (fd, CDIOREADTOCENTRY, &entry) == 0) {
-		    	if (entry.entry.control & CDROM_DATA_TRACK) {
+		    	if (!(entry.entry.control & CDROM_DATA_TRACK)) {
 			    	type = CDS_AUDIO;
 				break;
 			}
@@ -498,7 +507,7 @@ get_disc_type (const char *dev_path)
 		entries.starting_track = i;
 	    	entries.address_format = CD_LBA_FORMAT;
 		if (ioctl (fd, CDIOREADTOCENTRYS, &entries) == 0) {
-		    	if (entry.control & CDROM_DATA_TRACK) {
+		    	if (!(entry.control & CDROM_DATA_TRACK)) {
 			    	type = CDS_AUDIO;
 				break;
 			}
@@ -506,6 +515,7 @@ get_disc_type (const char *dev_path)
 	}
 
 #endif /* defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)*/
+	close (fd);
 	return type;
 #elif defined(__linux__)
 	fd = open (dev_path, O_RDONLY|O_NONBLOCK);
@@ -517,7 +527,11 @@ get_disc_type (const char *dev_path)
 		fd = -1;
 		return -1;
 	}
-	return ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
+
+	type = ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
+	close (fd);
+
+	return type;
 #else /* defined(__linux__) */
 	return CDS_NO_INFO;
 #endif /* defined(__linux__) */
@@ -627,8 +641,6 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 
 			return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 		}
-	} else {
-		g_warning ("Could not read MMC profile of %s", device);
 	}
 
 	close_ioctl_handle (ioctl_handle);
@@ -677,17 +689,17 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 }
 
 /**
- * nautilus_burn_drive_get_media_size_from_path:
+ * nautilus_burn_drive_get_media_type_from_path:
  * @device: Filename of drive device (eg. "/dev/hdb")
  *
- * Determine the capacity of the media in the drive at device @device.
+ * Determine the type of the media in the drive at device @device.
  *
- * Return value: The capcity in bytes of the media in the drive or the
+ * Return value: The #NautilusBurnMediaType of the media in the drive or the
  * following special values:
  *
- *    %NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN if the size can not be determined
- *    %NAUTILUS_BURN_MEDIA_SIZE_BUSY    if the device is busy
- *    %NAUTILUS_BURN_MEDIA_SIZE_NA      if the device is unknown
+ *    %NAUTILUS_BURN_MEDIA_TYPE_ERROR   if the type can not be determined
+ *    %NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN if the type can not be determined
+ *    %NAUTILUS_BURN_MEDIA_TYPE_BUSY    if the device is busy
  **/
 NautilusBurnMediaType
 nautilus_burn_drive_get_media_type_from_path (const char *device)
@@ -883,7 +895,12 @@ nautilus_burn_drive_get_media_type_full (NautilusBurnDrive *drive,
  *
  * Determine the capacity of the media in the specified device.
  *
- * Return value: The capacity of the media in bytes.
+ * Return value: The capacity of the media in bytes or the
+ * following special values:
+ *
+ *    %NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN if the type can not be determined
+ *    %NAUTILUS_BURN_MEDIA_SIZE_NA      if the device type is not recognized
+ *    %NAUTILUS_BURN_MEDIA_SIZE_BUSY    if the device is busy
  **/
 gint64
 nautilus_burn_drive_get_media_size_from_path (const char *device)
@@ -940,20 +957,107 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 	return size;
 }
 
+static gint64
+try_media_size_from_type (NautilusBurnDrive *drive)
+{
+	gint64                size = 0;
+	NautilusBurnMediaType type;
+	int                   fd;
+	int                   secs;
+	gpointer              ioctl_handle;
+
+	type = nautilus_burn_drive_get_media_type (drive);
+
+	ioctl_handle = open_ioctl_handle (drive->device);
+	if (ioctl_handle == INVALID_HANDLE) {
+		if (errno == EBUSY) {
+			return NAUTILUS_BURN_MEDIA_SIZE_BUSY;
+		}
+		return NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN;
+	}
+
+	fd = get_ioctl_handle_fd (ioctl_handle);
+
+	/* The CD types should be able to detect the media sizes
+	   the DVD types will likely fail since if we are here
+	   they probably aren't MMC capable */
+	switch (type) {
+	case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
+	case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
+	case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
+		size = NAUTILUS_BURN_MEDIA_SIZE_NA;
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_CD:
+		size = get_disc_size_cd (fd);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_CDR:
+	case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
+		secs = get_disc_size_cdr (fd);
+		if (secs > 0)
+			size = (1 + secs * 7 / 48) * 1024 * 1024;
+		else
+			size = 0;
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
+		size = get_disc_size_dvd (fd, 0x10);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
+		size = get_disc_size_dvd (fd, 0x11);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
+		size = get_disc_size_dvd (fd, 0x13);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
+		size = get_disc_size_dvd (fd, 0x12);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
+		size = get_disc_size_dvd (fd, 0x1B);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
+		size = get_disc_size_dvd (fd, 0x1A);
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
+		size = get_disc_size_dvd (fd, 0x2B);
+		break;
+	default:
+		break;
+	}
+
+	close_ioctl_handle (ioctl_handle);
+
+	return size;
+}
+
 /**
  * nautilus_burn_drive_get_media_size:
  * @drive: #NautilusBurnDrive
  *
  * Determine the capacity of the media in the specified drive.
  *
- * Return value: The capacity of the media in bytes.
+ * Return value: The capacity of the media in bytes or the
+ * following special values:
+ *
+ *    %NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN if the type can not be determined
+ *    %NAUTILUS_BURN_MEDIA_SIZE_NA      if the device type is not recognized
+ *    %NAUTILUS_BURN_MEDIA_SIZE_BUSY    if the device is busy
  **/
 gint64
 nautilus_burn_drive_get_media_size (NautilusBurnDrive *drive)
 {
+	gint64 size;
+
 	g_return_val_if_fail (drive != NULL, NAUTILUS_BURN_MEDIA_SIZE_UNKNOWN);
 
-	return nautilus_burn_drive_get_media_size_from_path (drive->device);
+#ifdef USE_HAL
+	size = nautilus_burn_drive_get_media_size_from_path (drive->device);
+	if (size < 0) {
+		size = try_media_size_from_type (drive);
+	}
+#else
+	size = nautilus_burn_drive_get_media_size_from_path (drive->device);
+#endif
+
+	return size;
 }
 
 typedef struct {
