@@ -1239,6 +1239,8 @@ nautilus_burn_recorder_run_process (NautilusBurnRecorder       *recorder,
 
 	local_error = NULL;
 
+	stdout_tag = 0;
+	stderr_tag = 0;
 	res = start_async_with_watch ((char **)argv->pdata,
 				      &process->pid,
 				      nautilus_burn_process_stdout_read,
@@ -1461,26 +1463,28 @@ nautilus_burn_recorder_write_cdrecord (NautilusBurnRecorder          *recorder,
 
 	track = tracks->data;
 
+	if (flags & NAUTILUS_BURN_RECORDER_WRITE_BLANK) {
+		NautilusBurnRecorderBlankFlags blank_flags = 0;
+		int                            res;
+		GError                        *err = NULL;
+
+		if (flags & NAUTILUS_BURN_RECORDER_WRITE_DEBUG)
+			blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DEBUG;
+		if (flags & NAUTILUS_BURN_RECORDER_WRITE_DUMMY_WRITE)
+			blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DUMMY_WRITE;
+		/* cdrdao doesn't seem to have on-the-fly blank */
+		/* cdrecord doesn't seem to correctly detect disc size when blanking */
+		res = nautilus_burn_recorder_blank_disc (recorder,
+							 drive,
+							 NAUTILUS_BURN_RECORDER_BLANK_FAST,
+							 blank_flags,
+							 &err);
+		if (res != NAUTILUS_BURN_RECORDER_RESULT_FINISHED)
+			return res;
+	}
+
+
 	if (track->type ==  NAUTILUS_BURN_RECORDER_TRACK_TYPE_CUE) {
-		if (flags & NAUTILUS_BURN_RECORDER_WRITE_BLANK) {
-			NautilusBurnRecorderBlankFlags blank_flags = 0;
-			int                            res;
-			GError                        *err = NULL;
-
-			if (flags & NAUTILUS_BURN_RECORDER_WRITE_DEBUG)
-				blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DEBUG;
-			if (flags & NAUTILUS_BURN_RECORDER_WRITE_DUMMY_WRITE)
-				blank_flags |= NAUTILUS_BURN_RECORDER_BLANK_DUMMY_WRITE;
-			/* cdrdao doesn't seem to have on-the-fly blank */
-			res = nautilus_burn_recorder_blank_disc (recorder,
-								 drive,
-								 NAUTILUS_BURN_RECORDER_BLANK_FAST,
-								 blank_flags,
-								 &err);
-			if (res != NAUTILUS_BURN_RECORDER_RESULT_FINISHED)
-				return res;
-		}
-
 		argv = g_ptr_array_new ();
 		g_ptr_array_add (argv, "cdrdao");
 		g_ptr_array_add (argv, "write");
@@ -1525,9 +1529,6 @@ nautilus_burn_recorder_write_cdrecord (NautilusBurnRecorder          *recorder,
 		if (flags & NAUTILUS_BURN_RECORDER_WRITE_EJECT) {
 			g_ptr_array_add (argv, "-eject");
 		}
-		if (flags & NAUTILUS_BURN_RECORDER_WRITE_BLANK) {
-			g_ptr_array_add (argv, "blank=fast");
-		}
 		if (flags & NAUTILUS_BURN_RECORDER_WRITE_DISC_AT_ONCE) {
 			g_ptr_array_add (argv, "-dao");
 		}
@@ -1820,6 +1821,9 @@ nautilus_burn_recorder_finalize (GObject *object)
 		nautilus_burn_process_free (recorder->priv->process);
 	}
 
+	g_free (recorder->priv);
+	recorder->priv = NULL;
+
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
